 /* -*- Mode: c -*-

     Lexer for a minimal programming language "nanoLang"

     Copyright 2015-2016 by Stephan Schulz, schulz@eprover.org.

     This code is released under the GNU General Public Licence
     Version 2.
 */

%option noyywrap
%option yylineno

%{
   #include <string.h>
   #include "ast.h"
   #include "lexemetable.h"
   #include "nanoparse.tab.h"   

   int ret_tok(int tok)
   {
      return tok;
   }

   int yycol = 0;

#define YY_USER_ACTION {yylloc.first_line = yylineno; yylloc.first_column = yycol; yycol += yyleng;}

%}

DIGIT      [0-9]
ALPHA      [a-zA-Z]
ALNUM      {ALPHA}|{DIGIT}

INT        {DIGIT}+
ID         {ALPHA}{ALNUM}*


%%

"("  {return ret_tok(OPENPAR);}
")"  {return ret_tok(CLOSEPAR);}
"*"  {return ret_tok(MULT);}
"/"  {return ret_tok(DIV);}
"+"  {return ret_tok(PLUS);}
"-"  {return ret_tok(MINUS);}

       
{ALPHA}{ALNUM}* {yylval = ASTAlloc2(t_IDENT, LexTableAddLexeme(&lextable, yytext), 
                                    0, NULL, NULL);
                 return ret_tok(IDENT);}

\"[^"]*\"       {yylval = ASTAlloc2(t_STRINGLIT, LexTableAddLexeme(&lextable, yytext),
                                    0, NULL, NULL);
                 return ret_tok(STRINGLIT);}

{DIGIT}+        {yylval = ASTAlloc2(t_INTLIT, LexTableAddLexeme(&lextable, yytext), 
                                    atoll(yytext), NULL, NULL);
                 return ret_tok(INTLIT);}


#[^\n]*\n { /* Eat comments */}

[\n]    {yycol=1;}

[\r\t ] { /* Skip  whitespace*/ }


 /* Everything else is an invalid character. */
.   {return ret_tok(ERROR);}

%%

